/*
 * Copyright (c) 2005 Performix, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Performix, Inc.("Confidential Information"). 
 * You shall not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into  with Performix, Inc.
 *
 * Performix, Inc. MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. Performix, Inc. SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *   
 */

package com.performix.module.integration.messagebus;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;

import com.performix.mesutil.logger.MesLogger;
import com.performix.module.configuration.MesEnvironment;
import com.performix.module.locale.MesDateTimeHandler;
import com.performix.schemas.core.MesAsyncMessageType;
import com.performix.schemas.core.MesELogEntryNotificationMsgUIType;
import com.performix.schemas.core.MesMiniPodUIType;

/**
 * 
 * xMES JMS Event Sender.
 * 
 * @author partha, gurpreet
 *
 */
public class EventMessageSender {

	private static Log log = MesLogger.getLogger(EventMessageSender.class);
    private String destinationTopic = "DestinationEvent"; //Can be changed from constructor
    private static final String moduleName = "MESSAGING";
    private static int defaultTTL = 1800000;
    private static int maxMessageLength = 2040; 
    
    /**
     * Default constructor
     *
     */
    public EventMessageSender() {
    	init();
    }
    /**
     * Constructor which sets contextFlag and attribute to read the topic name from mes.xml.
     * 
     * @param useDefaultContext
     * @param destinationTopicAttribute The attribute in mes.xml which holds the topic name.
     */
	public EventMessageSender(boolean useDefaultContext, String destinationTopicAttribute) {
		this.destinationTopic = destinationTopicAttribute ;
		init();
	}

	/**
	 * Inits the module
	 *
	 */
	private void init() {
    	try{
    		defaultTTL = Integer.parseInt(MesEnvironment.getValue("MESSAGING", "DefaultTimeToLiveForMessages"));
		}catch(Exception ex){
			log.warn("Unable to parse value of 'DefaultTimeToLiveForMessages' from mes.xml. Taking default TTL as: "+ defaultTTL);
		}
		
    	try{
    		maxMessageLength = Integer.parseInt(MesEnvironment.getValue("MESSAGING", "MaxLengthOfEventMessage"));
		}catch(Exception ex){
			log.warn("Unable to parse value of 'MaxLengthOfEventMessage' from mes.xml. Taking default length as: "+ maxMessageLength);
		}
				
	}
	
	/**
	 * Sends the AsynchMessages from list on JMS.
	 * 
	 * @param list
	 * @return
	 */
	public boolean sendMessages(List<MesAsyncMessageType> list ){
		String xml = AsyncMessageHelper.getXmlContentFromAsyncMessages(list);
		return JmsHelper.getInstance().sendMessage(xml, destinationTopic);
	}
	
	/**
	 * Sends the Async Message. 
	 * 
	 * @param message
	 * @return
	 */
	public boolean sendMessage(MesAsyncMessageType message ){
		if(message.getTtl() == 0){
			message.setTtl(defaultTTL);
		}
		if(message.getSourceTimeStamp()==null){
			message.setSourceTimeStamp(MesDateTimeHandler.getCalendar()); //Set to current time if user has forgot to set the timestamp.
		}
		List<MesAsyncMessageType> list = new ArrayList<MesAsyncMessageType>();
		String messageStr = message.getMessage();
		if(messageStr.length()>maxMessageLength){
			log.warn("Truncating event message to max length of "+ maxMessageLength);
			log.warn("Original Event Message: "+ messageStr);
			messageStr = messageStr.substring(0, maxMessageLength-1);
			message.setMessage(messageStr);
			log.warn("Truncated String : "+ messageStr);
		}
		list.add(message);
		return sendMessages (list);
	}

	/**
	 * Sends the MiniPod. 
	 * 
	 * @param MesMiniPodUIType
	 * @return
	 */
	public boolean sendMiniPod(MesMiniPodUIType miniPod ){
		List<MesMiniPodUIType> list = new ArrayList<MesMiniPodUIType>();
		list.add(miniPod);
		String xml = MiniPodSenderHelper.getXmlContentFromMiniPods(list);
		return sendSimpleMessage(xml);
	}

	/**
	 * 
	 * @param miniPod
	 * @return
	 */
	public boolean sendELogEntryNotificationMessage(MesAsyncMessageType elogEntryUI ){
		List<MesAsyncMessageType> list = new ArrayList<MesAsyncMessageType>();
		list.add(elogEntryUI);
		String xml = MiniPodSenderHelper.getXmlContentForELogEntryNotificationMsg(list);
		xml = "MessageELogBook" +xml;
		return sendSimpleMessage(xml);
	}
	/**
	 * Simple string posting method
	 * 
	 * @param message The message string to send.
	 * @return
	 */
	public boolean sendSimpleMessage(String message){
		return JmsHelper.getInstance().sendMessage(message, destinationTopic);
	}
	
	/**
	 * Closes JMS TOPIC
	 *
	 */
	public void close() {
		// Does nothing but left to legacy API calls. 
		//jmsTopicSender.closeAll();
	}
	
}
